"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findLargestTokenAccountForOwner = exports.getMultipleAccounts = exports.parseTokenAccountData = exports.ACCOUNT_LAYOUT = exports.getUnixTs = exports.promiseUndef = exports.getFilteredProgramAccounts = exports.nativeToUi = exports.uiToNative = exports.getMintDecimals = exports.createAccountInstruction = exports.awaitTransactionSignatureConfirmation = exports.simulateTransaction = exports.sleep = exports.zeroKey = void 0;
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js"));
const token_instructions_1 = require("@project-serum/serum/lib/token-instructions");
const buffer_layout_1 = require("buffer-layout");
const spl_token_1 = require("@solana/spl-token");
exports.zeroKey = new web3_js_1.PublicKey(new Uint8Array(32));
async function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
exports.sleep = sleep;
async function simulateTransaction(connection, transaction, commitment) {
    // @ts-ignore
    transaction.recentBlockhash = await connection._recentBlockhash(
    // @ts-ignore
    connection._disableBlockhashCaching);
    const signData = transaction.serializeMessage();
    // @ts-ignore
    const wireTransaction = transaction._serialize(signData);
    const encodedTransaction = wireTransaction.toString('base64');
    const config = { encoding: 'base64', commitment };
    const args = [encodedTransaction, config];
    // @ts-ignore
    const res = await connection._rpcRequest('simulateTransaction', args);
    if (res.error) {
        throw new Error('failed to simulate transaction: ' + res.error.message);
    }
    return res.result;
}
exports.simulateTransaction = simulateTransaction;
async function awaitTransactionSignatureConfirmation(txid, timeout, connection) {
    let done = false;
    const result = await new Promise((resolve, reject) => {
        (async () => {
            setTimeout(() => {
                if (done) {
                    return;
                }
                done = true;
                console.log('Timed out for txid', txid);
                reject({ timeout: true });
            }, timeout);
            try {
                connection.onSignature(txid, (result) => {
                    // console.log('WS confirmed', txid, result);
                    done = true;
                    if (result.err) {
                        reject(result.err);
                    }
                    else {
                        resolve(result);
                    }
                }, 'singleGossip');
                // console.log('Set up WS connection', txid);
            }
            catch (e) {
                done = true;
                console.log('WS error in setup', txid, e);
            }
            while (!done) {
                // eslint-disable-next-line no-loop-func
                (async () => {
                    try {
                        const signatureStatuses = await connection.getSignatureStatuses([
                            txid,
                        ]);
                        const result = signatureStatuses && signatureStatuses.value[0];
                        if (!done) {
                            if (!result) {
                                // console.log('REST null result for', txid, result);
                            }
                            else if (result.err) {
                                console.log('REST error for', txid, result);
                                done = true;
                                reject(result.err);
                            }
                            else if (!(result.confirmations || result.confirmationStatus === "confirmed" || result.confirmationStatus === "finalized")) {
                                console.log('REST not confirmed', txid, result);
                            }
                            else {
                                console.log('REST confirmed', txid, result);
                                done = true;
                                resolve(result);
                            }
                        }
                    }
                    catch (e) {
                        if (!done) {
                            console.log('REST connection error: txid', txid, e);
                        }
                    }
                })();
                await sleep(300);
            }
        })();
    });
    done = true;
    return result;
}
exports.awaitTransactionSignatureConfirmation = awaitTransactionSignatureConfirmation;
async function createAccountInstruction(connection, payer, space, owner, lamports) {
    const account = new web3_js_1.Account();
    const instruction = web3_js_1.SystemProgram.createAccount({
        fromPubkey: payer,
        newAccountPubkey: account.publicKey,
        lamports: lamports ? lamports : await connection.getMinimumBalanceForRentExemption(space),
        space,
        programId: owner
    });
    return { account, instruction };
}
exports.createAccountInstruction = createAccountInstruction;
const MINT_LAYOUT = buffer_layout_1.struct([buffer_layout_1.blob(44), buffer_layout_1.u8('decimals'), buffer_layout_1.blob(37)]);
async function getMintDecimals(connection, mint) {
    if (mint.equals(token_instructions_1.WRAPPED_SOL_MINT)) {
        return 9;
    }
    const { data } = throwIfNull(await connection.getAccountInfo(mint), 'mint not found');
    const { decimals } = MINT_LAYOUT.decode(data);
    return decimals;
}
exports.getMintDecimals = getMintDecimals;
function throwIfNull(value, message = 'account not found') {
    if (value === null) {
        throw new Error(message);
    }
    return value;
}
function uiToNative(amount, decimals) {
    return new bn_js_1.default(Math.round(amount * Math.pow(10, decimals)));
}
exports.uiToNative = uiToNative;
function nativeToUi(amount, decimals) {
    return amount / Math.pow(10, decimals);
}
exports.nativeToUi = nativeToUi;
async function getFilteredProgramAccounts(connection, programId, filters) {
    // @ts-ignore
    const resp = await connection._rpcRequest('getProgramAccounts', [
        programId.toBase58(),
        {
            commitment: connection.commitment,
            filters,
            encoding: 'base64',
        },
    ]);
    if (resp.error) {
        throw new Error(resp.error.message);
    }
    return resp.result.map(({ pubkey, account: { data, executable, owner, lamports } }) => ({
        publicKey: new web3_js_1.PublicKey(pubkey),
        accountInfo: {
            data: Buffer.from(data[0], 'base64'),
            executable,
            owner: new web3_js_1.PublicKey(owner),
            lamports,
        },
    }));
}
exports.getFilteredProgramAccounts = getFilteredProgramAccounts;
async function promiseUndef() {
    return undefined;
}
exports.promiseUndef = promiseUndef;
const getUnixTs = () => {
    return new Date().getTime() / 1000;
};
exports.getUnixTs = getUnixTs;
exports.ACCOUNT_LAYOUT = buffer_layout_1.struct([
    buffer_layout_1.blob(32, 'mint'),
    buffer_layout_1.blob(32, 'owner'),
    buffer_layout_1.nu64('amount'),
    buffer_layout_1.blob(93)
]);
function parseTokenAccountData(data) {
    let { mint, owner, amount } = exports.ACCOUNT_LAYOUT.decode(data);
    return {
        mint: new web3_js_1.PublicKey(mint),
        owner: new web3_js_1.PublicKey(owner),
        amount,
    };
}
exports.parseTokenAccountData = parseTokenAccountData;
async function getMultipleAccounts(connection, publicKeys) {
    const publickKeyStrs = publicKeys.map((pk) => (pk.toBase58()));
    // @ts-ignore
    const resp = await connection._rpcRequest('getMultipleAccounts', [publickKeyStrs]);
    if (resp.error) {
        throw new Error(resp.error.message);
    }
    return resp.result.value.map(({ data, executable, lamports, owner }, i) => ({
        publicKey: publicKeys[i],
        accountInfo: {
            data: Buffer.from(data[0], 'base64'),
            executable,
            owner: new web3_js_1.PublicKey(owner),
            lamports,
        },
    }));
}
exports.getMultipleAccounts = getMultipleAccounts;
async function findLargestTokenAccountForOwner(connection, owner, mint) {
    const response = await connection.getTokenAccountsByOwner(owner, { mint, programId: spl_token_1.TOKEN_PROGRAM_ID }, connection.commitment);
    let max = -1;
    let maxTokenAccount = null;
    let maxPubkey = null;
    for (const { pubkey, account } of response.value) {
        const tokenAccount = parseTokenAccountData(account.data);
        if (tokenAccount.amount > max) {
            maxTokenAccount = tokenAccount;
            max = tokenAccount.amount;
            maxPubkey = pubkey;
        }
    }
    if (maxPubkey && maxTokenAccount) {
        return { publicKey: maxPubkey, tokenAccount: maxTokenAccount };
    }
    else {
        throw new Error("No accounts for this token");
    }
}
exports.findLargestTokenAccountForOwner = findLargestTokenAccountForOwner;
//# sourceMappingURL=utils.js.map